@@ -9,21 +9,22 @@ from copy import deepcopy
 from argparse import Namespace
 from typing import Optional, List, Tuple, Dict, NewType, cast, Generator, Any, AsyncGenerator
 
-from bloom_filter import BloomFilter
+from bloom_filter2 import BloomFilter
 from typing_extensions import TypedDict, NotRequired, Protocol
 
 from shub_workflow.script import (
     JobKey,
     JobDict,
+    SpiderName,
     Outcome,
     BaseLoopScriptAsyncMixin,
 )
 from shub_workflow.base import WorkFlowManager
 from shub_workflow.utils import hashstr
+from shub_workflow.utils.dupefilter import DupesFilterProtocol
 
 
 _LOG = logging.getLogger(__name__)
-_LOG.setLevel(logging.INFO)
 
 
 SpiderArgs = NewType("SpiderArgs", Dict[str, str])
@@ -38,7 +39,7 @@ class JobParams(TypedDict):
 
 
 class FullJobParams(JobParams):
-    spider: NotRequired[str]
+    spider: NotRequired[SpiderName]
 
 
 def get_spider_args_from_params(params: JobParams) -> SpiderArgs:
@@ -66,14 +67,14 @@ class CrawlManager(WorkFlowManager):
     finished. Close reason of the manager will be inherited from spider one.
     """
 
-    spider: Optional[str] = None
+    spider: Optional[SpiderName] = None
 
     def __init__(self):
         super().__init__()
 
         # running jobs represents the state of a crawl manager.
         # a dict job key : (spider, job_args_override)
-        self._running_job_keys: Dict[JobKey, Tuple[str, JobParams]] = {}
+        self._running_job_keys: Dict[JobKey, Tuple[SpiderName, JobParams]] = {}
 
     @property
     def description(self):
@@ -102,7 +103,7 @@ class CrawlManager(WorkFlowManager):
     def schedule_spider_with_jobargs(
         self,
         job_args_override: Optional[JobParams] = None,
-        spider: Optional[str] = None,
+        spider: Optional[SpiderName] = None,
     ) -> Optional[JobKey]:
         job_args_override = job_args_override or {}
         spider = spider or self.spider
@@ -135,15 +136,20 @@ class CrawlManager(WorkFlowManager):
                     if job_args_override is not None:
                         job_args_override = job_args_override.copy()
                     self.bad_outcome_hook(spider, outcome, job_args_override, jobkey)
+                else:
+                    self.finished_ok_hook(spider, outcome, job_args_override, jobkey)
                 outcomes[jobkey] = outcome
         _LOG.info(f"There are {len(self._running_job_keys)} jobs still running.")
 
         return outcomes
 
-    def bad_outcome_hook(self, spider: str, outcome: str, job_args_override: JobParams, jobkey: JobKey):
+    def bad_outcome_hook(self, spider: SpiderName, outcome: Outcome, job_args_override: JobParams, jobkey: JobKey):
         if self.get_close_reason() is None:
             self.set_close_reason(outcome)
 
+    def finished_ok_hook(self, spider: SpiderName, outcome: Outcome, job_args_override: JobParams, jobkey: JobKey):
+        pass
+
     def workflow_loop(self) -> bool:
         outcomes = self.check_running_jobs()
         if outcomes:
@@ -172,7 +178,7 @@ class PeriodicCrawlManager(CrawlManager):
     parameters. Don't forget to set loop mode.
     """
 
-    def bad_outcome_hook(self, spider: str, outcome: str, job_args_override: JobParams, jobkey: JobKey):
+    def bad_outcome_hook(self, spider: str, outcome: Outcome, job_args_override: JobParams, jobkey: JobKey):
         pass
 
     def workflow_loop(self) -> bool:
@@ -187,9 +193,9 @@ class PeriodicCrawlManager(CrawlManager):
 
 class GeneratorCrawlManagerProtocol(Protocol):
 
-    _running_job_keys: Dict[JobKey, Tuple[str, JobParams]]
-    _jobuids: BloomFilter
-    spider: Optional[str]
+    _running_job_keys: Dict[JobKey, Tuple[SpiderName, JobParams]]
+    _jobuids: DupesFilterProtocol
+    spider: Optional[SpiderName]
 
     @property
     @abc.abstractmethod
@@ -208,7 +214,7 @@ class GeneratorCrawlManagerProtocol(Protocol):
     def schedule_spider_with_jobargs(
         self,
         job_args_override: Optional[JobParams] = None,
-        spider: Optional[str] = None,
+        spider: Optional[SpiderName] = None,
     ) -> Optional[JobKey]:
         ...
 
@@ -227,19 +233,47 @@ class GeneratorCrawlManager(CrawlManager, GeneratorCrawlManagerProtocol):
     - Don't forget to set loop mode and max jobs (which defaults to infinite).
     """
 
-    MAX_TOTAL_JOBS = 1000000
+    MAX_RETRIES = 0
+    max_jobs_per_spider = 1000
 
     def __init__(self):
         super().__init__()
         self.__parameters_gen = self.set_parameters_gen()
         self.__additional_jobs: List[FullJobParams] = []
+        self.__delayed_jobs: List[FullJobParams] = []
         self.__next_job_seq = 1
-        self._jobuids = BloomFilter(max_elements=self.MAX_TOTAL_JOBS, error_rate=0.001)
+        self._jobuids = self.create_dupe_filter()
 
-    def bad_outcome_hook(self, spider: str, outcome: str, job_args_override: JobParams, jobkey: JobKey):
-        pass
+    @classmethod
+    def create_dupe_filter(cls) -> DupesFilterProtocol:
+        return BloomFilter(max_elements=1e6, error_rate=1e-6)
 
-    def add_job(self, spider: str, job_args_override: JobParams):
+    def spider_running_count(self, spider: SpiderName) -> int:
+        count = 0
+        for spidername, _ in self._running_job_keys.values():
+            if spidername == spider:
+                count += 1
+        return count
+
+    def bad_outcome_hook(self, spider: SpiderName, outcome: Outcome, job_args_override: JobParams, jobkey: JobKey):
+        if outcome == "cancelled":
+            return
+        if self.MAX_RETRIES == 0:
+            return
+        spider_args = job_args_override.setdefault("spider_args", {})
+        tags = job_args_override["tags"]
+        retries = get_jobseq(tags)[1]
+        if retries < self.MAX_RETRIES:
+            for t in list(tags):
+                if t.startswith("RETRIED_FROM="):
+                    tags.remove(t)
+                    break
+            tags.append(f"RETRIED_FROM={jobkey}")
+            spider_args["retry_num"] = str(retries + 1)
+            self.add_job(spider, job_args_override)
+            _LOG.info(f"Job {jobkey} failed with reason '{outcome}'. Retrying ({retries + 1} of {self.MAX_RETRIES}).")
+
+    def add_job(self, spider: SpiderName, job_args_override: JobParams):
         params = cast(FullJobParams, (job_args_override or {}).copy())
         params["spider"] = spider
         self.__additional_jobs.append(params)
@@ -266,23 +300,44 @@ class GeneratorCrawlManager(CrawlManager, GeneratorCrawlManagerProtocol):
 
     def _workflow_step_gen(self, max_next_params: int) -> Generator[Tuple[str, Optional[JobKey]], None, None]:
         new_params = []
-        for i in range(max_next_params):
+        for _ in range(max_next_params):
             try:
-                if self.__additional_jobs:
+                if self.__delayed_jobs:
+                    next_params = self.__delayed_jobs.pop(0)
+                elif self.__additional_jobs:
                     next_params = self.__additional_jobs.pop(0)
                 else:
                     next_params = next(self.__parameters_gen)
                 new_params.append(next_params)
             except StopIteration:
                 break
+
+        running_counts: Dict[SpiderName, int] = {}
         for next_params in new_params:
-            spider = next_params.pop("spider", self.spider)
-            assert spider is not None, "No spider set."
+            spider = next_params.get("spider", self.spider)
+            assert spider, f"No spider set for parameters {next_params}"
+            next_params["spider"] = spider
+            if spider not in running_counts:
+                running_counts[spider] = self.spider_running_count(spider)
+
+        for next_params in new_params:
+            if running_counts.setdefault(next_params["spider"], 0) >= self.get_max_jobs_per_spider(
+                next_params["spider"]
+            ):
+                self.__delayed_jobs.append(next_params)
+                continue
+            spider = next_params.pop("spider")
+            running_counts[spider] += 1
             spider_args = get_spider_args_from_params(next_params)
             jobuid = self.get_job_unique_id({"spider": spider, "spider_args": spider_args})
             if jobuid not in self._jobuids:
                 self.__add_jobseq_tag(next_params)
                 yield jobuid, self.schedule_spider_with_jobargs(next_params, spider)
+            else:
+                _LOG.warning(f"Job with spider {spider} and parameters {next_params} was already scheduled. Skipped.")
+
+    def get_max_jobs_per_spider(self, spider: SpiderName) -> int:
+        return self.max_jobs_per_spider
 
     def workflow_loop(self) -> bool:
         self.check_running_jobs()
@@ -320,7 +375,7 @@ class GeneratorCrawlManager(CrawlManager, GeneratorCrawlManagerProtocol):
         _LOG.info(f"Next job sequence number: {self.__next_job_seq}")
 
     def on_close(self):
-        pass
+        self._jobuids.close()
 
 
 class AsyncSchedulerCrawlManagerMixin(BaseLoopScriptAsyncMixin, GeneratorCrawlManagerProtocol):
@@ -345,7 +400,7 @@ class AsyncSchedulerCrawlManagerMixin(BaseLoopScriptAsyncMixin, GeneratorCrawlMa
     async def schedule_spider_with_jobargs(  # type: ignore
         self,
         job_args_override: Optional[JobParams] = None,
-        spider: Optional[str] = None,
+        spider: Optional[SpiderName] = None,
     ) -> Optional[JobKey]:
         job_args_override = job_args_override or {}
         spider = spider or self.spider
