@@ -1,7 +1,7 @@
 import re
 import logging
 import uuid
-from typing import List, Generator
+from typing import List, Generator, Callable
 from operator import itemgetter
 from glob import iglob
 from os import listdir, remove, environ, makedirs
@@ -21,7 +21,6 @@ BUFFER = 1024 * 1024
 
 
 logger = logging.getLogger(__name__)
-logger.setLevel(logging.INFO)
 
 
 def just_log_exception(exception):
@@ -84,38 +83,47 @@ UPLOAD_CHUNK_SIZE = 100 * 1024 * 1024
 
 
 def download_file(path, dest=None, aws_key=None, aws_secret=None, aws_token=None, **kwargs):
-    assert path.startswith(_S3_ATTRIBUTE), f"Not a s3 source: {path}"
     if dest is None:
         dest = basename(path)
-    with get_file(path, "rb", aws_key=aws_key, aws_secret=aws_secret, aws_token=aws_token, **kwargs) as r, open(
-        dest, "wb"
-    ) as w:
-        total = 0
-        while True:
-            size = w.write(r.read(DOWNLOAD_CHUNK_SIZE))
-            total += size
-            logger.info(f"Downloaded {total} bytes from {path}")
-            if size < DOWNLOAD_CHUNK_SIZE:
-                break
+    if path.startswith(_S3_ATTRIBUTE):
+        with get_file(path, "rb", aws_key=aws_key, aws_secret=aws_secret, aws_token=aws_token, **kwargs) as r, open(
+            dest, "wb"
+        ) as w:
+            total = 0
+            while True:
+                size = w.write(r.read(DOWNLOAD_CHUNK_SIZE))
+                total += size
+                logger.info(f"Downloaded {total} bytes from {path}")
+                if size < DOWNLOAD_CHUNK_SIZE:
+                    break
+    elif path.startswith(_GS_ATTRIBUTE):
+        gcstorage.download_file(path, dest)
+    else:
+        raise ValueError(f"Not supported file system fpr path {path}")
 
 
 def upload_file_obj(robj, dest, aws_key=None, aws_secret=None, aws_token=None, **kwargs):
-    assert dest.startswith(_S3_ATTRIBUTE), f"Not a s3 source: {dest}"
-    with get_file(dest, "wb", aws_key=aws_key, aws_secret=aws_secret, aws_token=aws_token, **kwargs) as w:
-        total = 0
-        while True:
-            size = w.write(robj.read(UPLOAD_CHUNK_SIZE))
-            total += size
-            logger.info(f"Uploaded {total} bytes to {dest}")
-            if size < UPLOAD_CHUNK_SIZE:
-                break
+    if dest.startswith(_S3_ATTRIBUTE):
+        with get_file(dest, "wb", aws_key=aws_key, aws_secret=aws_secret, aws_token=aws_token, **kwargs) as w:
+            total = 0
+            while True:
+                size = w.write(robj.read(UPLOAD_CHUNK_SIZE))
+                total += size
+                logger.info(f"Uploaded {total} bytes to {dest}")
+                if size < UPLOAD_CHUNK_SIZE:
+                    break
+    else:
+        raise ValueError("Not a supported cloud.")
 
 
 def upload_file(path, dest, aws_key=None, aws_secret=None, aws_token=None, **kwargs):
     if dest.endswith("/"):
         dest = dest + basename(path)
     with open(path, "rb") as r:
-        upload_file_obj(r, dest, aws_key, aws_secret, aws_token, **kwargs)
+        if dest.startswith(_S3_ATTRIBUTE):
+            upload_file_obj(r, dest, aws_key, aws_secret, aws_token, **kwargs)
+        elif dest.startswith(_GS_ATTRIBUTE):
+            gcstorage.upload_file(path, dest)
 
 
 def get_glob(path, aws_key=None, aws_secret=None, aws_token=None, **kwargs):
@@ -166,11 +174,13 @@ def rm_file(path, aws_key=None, aws_secret=None, aws_token=None, **kwargs):
         op_kwargs = kwargs.pop("op_kwargs", {})
         fs = S3FileSystem(**s3_credentials(aws_key, aws_secret, aws_token, region), **kwargs)
         fs.rm(s3_path(path), **op_kwargs)
+    elif path.startswith(_GS_ATTRIBUTE):
+        gcstorage.rm_file(path)
     else:
         remove(path)
 
 
-def list_path(path, aws_key=None, aws_secret=None, aws_token=None, **kwargs):
+def list_path(path, aws_key=None, aws_secret=None, aws_token=None, **kwargs) -> Generator[str, None, None]:
     """
     More efficient boto3 based path listing, that accepts prefix
     """
@@ -184,17 +194,23 @@ def list_path(path, aws_key=None, aws_secret=None, aws_token=None, **kwargs):
     )
     if path.startswith(_S3_ATTRIBUTE):
         s3 = session.resource("s3")
-        bucket_name, path = S3_PATH_RE.match(path).groups()
-        bucket = s3.Bucket(bucket_name)
-        for result in bucket.objects.filter(Prefix=path):
-            yield f"s3://{bucket_name}/{result.key}"
+        m = S3_PATH_RE.match(path)
+        if m:
+            bucket_name, path = m.groups()
+            bucket = s3.Bucket(bucket_name)
+            for result in bucket.objects.filter(Prefix=path):
+                yield f"s3://{bucket_name}/{result.key}"
+        else:
+            raise ValueError(f"Bad s3 path specification: {path}")
+    elif path.startswith(_GS_ATTRIBUTE):
+        yield from list_folder(path)
     else:
         raise NotImplementedError("Only implemented for s3.")
 
 
 def list_folder(path, aws_key=None, aws_secret=None, aws_token=None, **kwargs) -> List[str]:
-    region = kwargs.pop("region", None)
     if path.startswith(_S3_ATTRIBUTE):
+        region = kwargs.pop("region", None)
         fs = S3FileSystem(**s3_credentials(aws_key, aws_secret, aws_token, region), **kwargs)
 
         try:
@@ -202,7 +218,8 @@ def list_folder(path, aws_key=None, aws_secret=None, aws_token=None, **kwargs) -
             listing = [f"s3://{name}" for name in fs.ls(path) if name != path]
         except FileNotFoundError:
             listing = []
-
+    elif path.startswith(_GS_ATTRIBUTE):
+        listing = list(gcstorage.list_folder(path))
     else:
         if path.strip().endswith("/"):
             path = path[:-1]
@@ -332,6 +349,24 @@ class S3SessionFactory:
 
 
 class S3Helper:
+
+    cp_file: Callable
+    download_file: Callable
+    empty_folder: Callable
+    exists: Callable
+    get_file: Callable
+    get_glob: Callable
+    list_folder: Callable
+    list_folder_files_recursive: Callable
+    list_folder_in_ts_order: Callable
+    list_path: Callable
+    mv_file: Callable
+    rm_file: Callable
+    s3_folder_size: Callable
+    touch: Callable
+    upload_file: Callable
+    upload_file_obj: Callable
+
     def __init__(self, aws_key, aws_secret, aws_role=None, aws_external_id=None, expiration_margin_minutes=10):
         self.s3_session_factory = None
         self.credentials = {"aws_key": aws_key, "aws_secret": aws_secret}
@@ -340,25 +375,10 @@ class S3Helper:
                 aws_key, aws_secret, aws_role, aws_external_id, expiration_margin_minutes
             )
 
-        for method in (
-            get_file,
-            upload_file_obj,
-            upload_file,
-            download_file,
-            list_folder,
-            mv_file,
-            rm_file,
-            get_glob,
-            empty_folder,
-            exists,
-            touch,
-            cp_file,
-            list_path,
-            list_folder_in_ts_order,
-            list_folder_files_recursive,
-            s3_folder_size,
-        ):
-            self._wrap_method(method)
+        for method_name, _type in S3Helper.__annotations__.items():
+            if not hasattr(self, method_name) and _type is Callable:
+                method = globals()[method_name]
+                self._wrap_method(method)
 
     def _wrap_method(self, method):
         def _method(*args, **kwargs):
