@@ -28,16 +28,16 @@ from shub_workflow.utils import (
 
 
 logger = logging.getLogger(__name__)
-logger.setLevel(logging.INFO)
 
 
 JobKey = NewType("JobKey", str)  # ScrapyCloud job key
 Outcome = NewType("Outcome", str)
+SpiderName = NewType("SpiderName", str)
 
 
 class JobDict(TypedDict):
 
-    spider: str
+    spider: SpiderName
     key: NotRequired[JobKey]
     tags: NotRequired[List[str]]
     close_reason: NotRequired[str]
@@ -152,7 +152,17 @@ class BaseScriptProtocol(ArgumentParserScriptProtocol, Protocol):
         ...
 
 
-class BaseScript(ArgumentParserScript, BaseScriptProtocol):
+class SCProjectClass:
+    def __init__(self):
+        self.project_id: Optional[int] = resolve_project_id()
+        self.client = ScrapinghubClient(max_retries=100)
+        super().__init__()
+
+    def get_project(self, project_id: Optional[Union[int, str]] = None) -> Project:
+        return self.client.get_project(project_id or self.project_id)
+
+
+class BaseScript(SCProjectClass, ArgumentParserScript, BaseScriptProtocol):
 
     name = ""  # optional, may be needed for some applications
     flow_id_required = False  # if True, script can only run in the context of a flow_id
@@ -160,12 +170,10 @@ class BaseScript(ArgumentParserScript, BaseScriptProtocol):
     default_project_id: Optional[int] = None  # If None, autodetect (see shub_workflow.utils.resolve_project_id)
 
     def __init__(self):
-        self.project_id: Optional[int] = None
-        self.client = ScrapinghubClient(max_retries=100)
         self.close_reason: Optional[str] = None
         self.__flow_tags: List[str] = []
         self.project_settings = get_project_settings()
-        self.loader = SpiderLoader(self.project_settings)
+        self.spider_loader = SpiderLoader(self.project_settings)
         super().__init__()
         self.set_flow_id_name(self.args)
 
@@ -219,9 +227,6 @@ class BaseScript(ArgumentParserScript, BaseScriptProtocol):
 
         return args
 
-    def get_project(self, project_id: Optional[Union[int, str]] = None) -> Project:
-        return self.client.get_project(project_id or self.project_id)
-
     def get_own_jobkey_from_env(self) -> Optional[JobKey]:
         envjobkey = os.getenv("SHUB_JOBKEY")
         if envjobkey is not None:
@@ -294,6 +299,21 @@ class BaseScript(ArgumentParserScript, BaseScriptProtocol):
                 if metadata:
                     self._update_metadata(metadata, {"tags": job_tags})
 
+    def remove_job_tags(self, tags: List[str], jobkey: JobKey):
+        update = False
+        job_tags = self.get_job_tags(jobkey)
+        for tag in tags:
+            if tag in job_tags:
+                if any([tag.startswith(s) for s in ("FLOW_ID=", "NAME=", "PARENT_NAME=")]):
+                    logger.info(f"Cannot remove tag {tag}: not allowed.")
+                else:
+                    job_tags.remove(tag)
+                    update = True
+        if update:
+            metadata = self.get_job_metadata(jobkey)
+            if metadata:
+                self._update_metadata(metadata, {"tags": job_tags})
+
     async def async_add_job_tags(self, jobkey: Optional[JobKey] = None, tags: Optional[List[str]] = None):
         loop = asyncio.get_event_loop()
         return await loop.run_in_executor(None, partial(self.add_job_tags, jobkey, tags))
@@ -330,6 +350,7 @@ class BaseScript(ArgumentParserScript, BaseScriptProtocol):
         except DuplicateJobError as e:
             logger.error(str(e))
         except Exception:
+            logger.error(f"Failed to schedule job with arguments {schedule_kwargs}")
             raise
         else:
             logger.info(f"Scheduled job {job.key}")
@@ -457,6 +478,7 @@ class BaseLoopScript(BaseScript, BaseLoopScriptProtocol):
     # --loop-mode command line option overrides it
     loop_mode = 0
     max_running_time = 0
+    stats_interval = 120
 
     def __init__(self):
         self.workflow_loop_enabled = False
@@ -471,6 +493,7 @@ class BaseLoopScript(BaseScript, BaseLoopScriptProtocol):
         logger.debug(f"Stats collection class: {stats_collector_class}")
         self.stats = load_object(stats_collector_class)(PseudoCrawler(self))
         self.__close_reason = None
+        self.__last_stats_upload = None
 
     def set_close_reason(self, reason):
         self.__close_reason = reason
@@ -531,11 +554,23 @@ class BaseLoopScript(BaseScript, BaseLoopScriptProtocol):
             except KeyboardInterrupt:
                 logger.info("Bye")
                 break
+            now = time.time()
+            if self.__last_stats_upload is None or now - self.__last_stats_upload >= self.stats_interval:
+                self.maybe_upload_stats()
+                self.__last_stats_upload = now
+
+    def maybe_upload_stats(self):
+        """
+        this is a bit dirty, but it is needed for HubStorageStatsCollector and
+        there is not a base upload method in base stats class. Will improve
+        in future.
+        """
+        if hasattr(self.stats, "_upload_stats"):
+            self.stats._upload_stats()
 
     def _on_start(self):
         self.on_start()
         self.workflow_loop_enabled = True
-        self.stats.open_spider(self)
 
     def on_close(self):
         pass
@@ -543,7 +578,7 @@ class BaseLoopScript(BaseScript, BaseLoopScriptProtocol):
     def _close(self):
         self.on_close()
         self.__close_reason = self.__close_reason or "finished"
-        self.stats.close_spider(self, self.__close_reason)
+        self.maybe_upload_stats()
 
 
 class BaseLoopScriptAsyncMixin(BaseLoopScriptProtocol):
