@@ -7,6 +7,7 @@ from typing import Tuple
 from unittest import TestCase
 from unittest.mock import patch, Mock, call
 
+from shub_workflow.script import Outcome
 from shub_workflow.graph import GraphManager
 from shub_workflow.graph.task import Task, BaseTask, SpiderTask, Resource, ResourcesDict, TaskId, GraphManagerProtocol
 from shub_workflow.utils.contexts import script_args
@@ -160,13 +161,13 @@ class ManagerTest(BaseTestCase):
         self.assertFalse(manager.schedule_script.called)
 
         # fourth loop, jobA finished (but has to wait for jobB for next job)
-        manager.is_finished = lambda x: "finished" if x == "999/1/3" else None
+        manager.is_finished = lambda x: Outcome("finished") if x == "999/1/3" else None
         result = next(manager._run_loops())
         self.assertTrue(result)
         self.assertFalse(manager.schedule_script.called)
 
         # fifth loop, jobB finished, now we can run jobC
-        manager.is_finished = lambda x: "finished" if x == "999/1/2" else None
+        manager.is_finished = lambda x: Outcome("finished") if x == "999/1/2" else None
         manager.schedule_script.side_effect = ["999/1/4"]
         result = next(manager._run_loops())
         self.assertTrue(result)
@@ -176,7 +177,7 @@ class ManagerTest(BaseTestCase):
         )
 
         # sixth loop, jobC finished, schedule jobD
-        manager.is_finished = lambda x: "finished" if x == "999/1/4" else None
+        manager.is_finished = lambda x: Outcome("finished") if x == "999/1/4" else None
         manager.schedule_script.reset_mock()
         manager.schedule_script.side_effect = ["999/1/5"]
         result = next(manager._run_loops())
@@ -187,7 +188,7 @@ class ManagerTest(BaseTestCase):
         )
 
         # last loop, jobD finished, workflow finished
-        manager.is_finished = lambda x: "finished" if x == "999/1/5" else None
+        manager.is_finished = lambda x: Outcome("finished") if x == "999/1/5" else None
         manager.schedule_script.reset_mock()
         result = next(manager._run_loops())
         self.assertFalse(result)
@@ -491,7 +492,7 @@ class ManagerTest(BaseTestCase):
         )
 
         # fourth loop, job A finishes, will start now parallel job B
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         manager.schedule_script.side_effect = [
             "999/2/1",
             "999/2/2",
@@ -509,14 +510,14 @@ class ManagerTest(BaseTestCase):
             )
 
         # fifth loop, jobB finishes partially, neither jobD nor jobC can still be scheduled
-        manager.is_finished = lambda x: "finished" if x == "999/2/1" else None
+        manager.is_finished = lambda x: Outcome("finished") if x == "999/2/1" else None
         manager.schedule_script.reset_mock()
         result = next(manager._run_loops())
         self.assertTrue(result)
         self.assertFalse(manager.schedule_script.called)
 
         # sixth loop, jobB finishes, jobD now can run, also jobC is scheduled
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         manager.schedule_script.side_effect = ["999/3/1", "999/3/2"]
         result = next(manager._run_loops())
         self.assertTrue(result)
@@ -557,25 +558,25 @@ class ManagerTest(BaseTestCase):
             )
 
         # second loop one of the subtasks finished
-        manager.is_finished = lambda x: "finished" if x == "999/1/2" else None
+        manager.is_finished = lambda x: Outcome("finished") if x == "999/1/2" else None
         result = next(manager._run_loops())
         self.assertTrue(result)
         self.assertTrue(manager.schedule_script.call_count, 0)
 
         # third loop a second subtask finished
-        manager.is_finished = lambda x: "finished" if x == "999/1/1" else None
+        manager.is_finished = lambda x: Outcome("finished") if x == "999/1/1" else None
         result = next(manager._run_loops())
         self.assertTrue(result)
         self.assertTrue(manager.schedule_script.call_count, 0)
 
         # fourth loop a third subtask finished
-        manager.is_finished = lambda x: "finished" if x == "999/1/4" else None
+        manager.is_finished = lambda x: Outcome("finished") if x == "999/1/4" else None
         result = next(manager._run_loops())
         self.assertTrue(result)
         self.assertTrue(manager.schedule_script.call_count, 0)
 
         # fifth loop, all job A finishes, will start now parallel job B
-        manager.is_finished = lambda x: "finished" if x == "999/1/3" else None
+        manager.is_finished = lambda x: Outcome("finished") if x == "999/1/3" else None
         manager.schedule_script.side_effect = [
             "999/2/1",
             "999/2/2",
@@ -638,7 +639,7 @@ class ManagerTest(BaseTestCase):
         )
 
         # third loop, all job A finishes, will start now parallel job B
-        manager.is_finished = lambda x: "finished" if x == "999/1/5" else None
+        manager.is_finished = lambda x: Outcome("finished") if x == "999/1/5" else None
         manager.schedule_script.side_effect = [
             "999/2/1",
             "999/2/2",
@@ -834,7 +835,7 @@ class ManagerTest(BaseTestCase):
             )
 
         # second loop, jobA finishes, jobB is scheduled, not jobC or next ones as it was skipped
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         manager.schedule_script.side_effect = [
             "999/2/1",
             "999/2/2",
@@ -852,7 +853,7 @@ class ManagerTest(BaseTestCase):
             )
 
         # last loop, jobB finished, workflow finished
-        manager.is_finished = lambda x: "finished" if x.startswith("999/2") else None
+        manager.is_finished = lambda x: Outcome("finished") if x.startswith("999/2") else None
         manager.schedule_script.reset_mock()
         result = next(manager._run_loops())
         self.assertFalse(result)
@@ -887,7 +888,7 @@ class ManagerTest(BaseTestCase):
             )
 
         # second loop, jobA finishes, jobB and jobC are scheduled
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         manager.schedule_script.side_effect = [
             "999/2/1",
             "999/2/2",
@@ -909,21 +910,21 @@ class ManagerTest(BaseTestCase):
         )
 
         # third loop, jobB finishes
-        manager.is_finished = lambda x: "finished" if x.startswith("999/2/") else None
+        manager.is_finished = lambda x: Outcome("finished") if x.startswith("999/2/") else None
         manager.schedule_script.reset_mock()
         result = next(manager._run_loops())
         self.assertTrue(result)
         self.assertFalse(manager.schedule_script.called)
 
         # fourth loop, jobC finishes, jobD is scheduled
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         manager.schedule_script.side_effect = ["999/4/1"]
         result = next(manager._run_loops())
         self.assertTrue(result)
         manager.schedule_script.assert_any_call(["commandD"], tags=["TASK_ID=jobD"], units=None, project_id=None)
 
         # fifth loop, jobD finishes, jobE is scheduled
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         manager.schedule_script.side_effect = ["999/5/1"]
         result = next(manager._run_loops())
         self.assertTrue(result)
@@ -947,7 +948,7 @@ class ManagerTest(BaseTestCase):
         # second loop, jobD finishes, jobE is scheduled, regardless it should wait for jobB to finish.
         # However, as defined in the start jobs, jobB will never be scheduled (i.e. could have been
         # already scheduled/finished by another instance of the manager)
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         manager.schedule_script.side_effect = ["999/5/1"]
         result = next(manager._run_loops())
         self.assertTrue(result)
@@ -982,7 +983,7 @@ class ManagerTest(BaseTestCase):
             )
 
         # second loop, jobA finishes, jobC is scheduled, but not jobB as it is skipped.
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         manager.schedule_script.side_effect = ["999/3/1"]
         result = next(manager._run_loops())
         self.assertTrue(result)
@@ -991,14 +992,14 @@ class ManagerTest(BaseTestCase):
         )
 
         # third loop, jobC finishes, jobD is scheduled
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         manager.schedule_script.side_effect = ["999/4/1"]
         result = next(manager._run_loops())
         self.assertTrue(result)
         manager.schedule_script.assert_any_call(["commandD"], tags=["TASK_ID=jobD"], units=None, project_id=None)
 
         # fourth loop, jobD finishes, jobE is scheduled (will not wait for B as it was skipped)
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         manager.schedule_script.side_effect = ["999/5/1"]
         result = next(manager._run_loops())
         self.assertTrue(result)
@@ -1031,7 +1032,7 @@ class ManagerTest(BaseTestCase):
             )
 
         # second loop, jobB finishes, jobE is scheduled
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         manager.schedule_script.side_effect = ["999/5/1"]
         result = next(manager._run_loops())
         self.assertTrue(result)
@@ -1064,7 +1065,7 @@ class ManagerTest(BaseTestCase):
             )
 
         # second loop, one jobB finishes, jobC must not still be scheduled
-        manager.is_finished = lambda x: "finished" if x == "999/2/1" else None
+        manager.is_finished = lambda x: Outcome("finished") if x == "999/2/1" else None
         manager.schedule_script.reset_mock()
         result = next(manager._run_loops())
         self.assertTrue(result)
@@ -1123,14 +1124,14 @@ class ManagerTest(BaseTestCase):
         )
 
         # second loop, jobC finishes, jobD is scheduled
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         manager.schedule_script.side_effect = ["999/4/1"]
         result = next(manager._run_loops())
         self.assertTrue(result)
         manager.schedule_script.assert_any_call(["commandD"], tags=["TASK_ID=jobD"], units=None, project_id=None)
 
         # third loop, jobD finishes, jobE is scheduled
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         manager.schedule_script.side_effect = ["999/5/1"]
         result = next(manager._run_loops())
         self.assertTrue(result)
@@ -1199,7 +1200,7 @@ class ManagerTest(BaseTestCase):
             )
 
         # second loop, jobB finishes, jobC is scheduled
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         manager.schedule_script.side_effect = ["999/3/1"]
         result = next(manager._run_loops())
         self.assertTrue(result)
@@ -1208,14 +1209,14 @@ class ManagerTest(BaseTestCase):
         )
 
         # third loop, jobC finishes, jobD is scheduled
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         manager.schedule_script.side_effect = ["999/4/1"]
         result = next(manager._run_loops())
         self.assertTrue(result)
         manager.schedule_script.assert_any_call(["commandD"], tags=["TASK_ID=jobD"], units=None, project_id=None)
 
         # fourth loop, jobD finishes, jobE is scheduled
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         manager.schedule_script.side_effect = ["999/5/1"]
         result = next(manager._run_loops())
         self.assertTrue(result)
@@ -1284,7 +1285,7 @@ class ManagerTest(BaseTestCase):
             )
 
         # second loop, jobB finishes, jobC must be scheduled, but not jobD
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         manager.schedule_script.side_effect = ["999/3/1"]
         result = next(manager._run_loops())
         self.assertTrue(result)
@@ -1293,14 +1294,14 @@ class ManagerTest(BaseTestCase):
         )
 
         # third loop, jobC finishes, jobD is scheduled, but not jobE
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         manager.schedule_script.side_effect = ["999/4/1"]
         result = next(manager._run_loops())
         self.assertTrue(result)
         manager.schedule_script.assert_any_call(["commandD"], tags=["TASK_ID=jobD"], units=None, project_id=None)
 
         # fourth loop, jobD finishes, jobE is scheduled
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         manager.schedule_script.side_effect = ["999/5/1"]
         result = next(manager._run_loops())
         self.assertTrue(result)
@@ -1369,7 +1370,7 @@ class ManagerTest(BaseTestCase):
             )
 
         # second loop, jobB finishes, jobC must be scheduled, but not jobE
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         manager.schedule_script.side_effect = ["999/3/1"]
         result = next(manager._run_loops())
         self.assertTrue(result)
@@ -1378,14 +1379,14 @@ class ManagerTest(BaseTestCase):
         )
 
         # third loop, jobC finishes, jobE is scheduled, but not jobD
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         manager.schedule_script.side_effect = ["999/5/1"]
         result = next(manager._run_loops())
         self.assertTrue(result)
         manager.schedule_script.assert_any_call(["commandE"], tags=["TASK_ID=jobE"], units=None, project_id=None)
 
         # fourth loop, jobE finishes, jobD is scheduled
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         manager.schedule_script.side_effect = ["999/4/1"]
         result = next(manager._run_loops())
         self.assertTrue(result)
@@ -1448,7 +1449,7 @@ class ManagerTest(BaseTestCase):
         self.assertTrue(next(manager._run_loops()))
 
         # third loop, jobs A finishes, jobB scheduled
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         manager.schedule_script.side_effect = ["999/2/1"]
         self.assertTrue(next(manager._run_loops()))
         manager.schedule_script.assert_called_with(
@@ -1460,7 +1461,7 @@ class ManagerTest(BaseTestCase):
         self.assertTrue(next(manager._run_loops()))
 
         # 5th loop, job B finishes, jobC scheduled
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         manager.schedule_script.side_effect = ["999/3/1"]
         self.assertTrue(next(manager._run_loops()))
         manager.schedule_script.assert_called_with(
@@ -1507,7 +1508,7 @@ class ManagerTest(BaseTestCase):
         self.assertTrue(result)
         manager.schedule_script.assert_any_call(["commandA"], tags=["TASK_ID=jobA"], units=None, project_id=None)
 
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         with self.assertRaisesRegex(
             RuntimeError,
             re.escape(
@@ -1567,7 +1568,7 @@ class ManagerTest(BaseTestCase):
         self.assertTrue(result)
         self.assertEqual(manager.schedule_script.call_count, 0)
 
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         result = next(manager._run_loops())
         self.assertTrue(result)
         self.assertEqual(manager.schedule_script.call_count, 1)
@@ -1623,7 +1624,7 @@ class ManagerTest(BaseTestCase):
         self.assertEqual(manager.schedule_script.call_count, 0)
 
         # third loop: commandC finishes so commandD can run
-        manager.is_finished = lambda x: "finished" if x == "999/3/1" else None
+        manager.is_finished = lambda x: Outcome("finished") if x == "999/3/1" else None
         manager.schedule_script.side_effect = ["999/4/1"]
         result = next(manager._run_loops())
         self.assertTrue(result)
@@ -1632,13 +1633,13 @@ class ManagerTest(BaseTestCase):
 
         # fourth loop: commandD finishes. commandB, can't still run
         manager.schedule_script.reset_mock()
-        manager.is_finished = lambda x: "finished" if x == "999/4/1" else None
+        manager.is_finished = lambda x: Outcome("finished") if x == "999/4/1" else None
         result = next(manager._run_loops())
         self.assertTrue(result)
         self.assertEqual(manager.schedule_script.call_count, 0)
 
         # fifth loop: commandA finishes so commandB can run
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         manager.schedule_script.side_effect = ["999/2/1"]
         result = next(manager._run_loops())
         self.assertTrue(result)
@@ -1699,7 +1700,7 @@ class ManagerTest(BaseTestCase):
         self.assertEqual(manager.schedule_script.call_count, 0)
 
         # Third loop. All jobA subjobs finished. Schedule jobB.
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         result = next(manager._run_loops())
         self.assertTrue(result)
         self.assertEqual(manager.schedule_script.call_count, 1)
@@ -1755,7 +1756,7 @@ class ManagerTest(BaseTestCase):
         manager.schedule_script.assert_has_calls(calls)
 
         # If only jobA is finished, jobC can't run
-        manager.is_finished = lambda x: "finished" if x.startswith("999/1/") else None
+        manager.is_finished = lambda x: Outcome("finished") if x.startswith("999/1/") else None
         manager.schedule_script.reset_mock()
 
         result = next(manager._run_loops())
@@ -1763,7 +1764,7 @@ class ManagerTest(BaseTestCase):
         self.assertEqual(manager.schedule_script.call_count, 0)
 
         # finished jobB, jobC can run
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         manager.schedule_script.side_effect = [f"999/3/{i}" for i in range(1, 5)]
         result = next(manager._run_loops())
         self.assertTrue(result)
@@ -1824,7 +1825,7 @@ class ManagerTest(BaseTestCase):
         manager.schedule_script.assert_has_calls(calls)
 
         # If any job is finished, jobC can run
-        manager.is_finished = lambda x: "finished" if x.startswith("999/1/") else None
+        manager.is_finished = lambda x: Outcome("finished") if x.startswith("999/1/") else None
         manager.schedule_script.reset_mock()
 
         manager.schedule_script.side_effect = [f"999/3/{i}" for i in range(1, 5)]
@@ -1882,7 +1883,7 @@ class ManagerTest(BaseTestCase):
         manager.schedule_script.assert_has_calls(calls)
 
         # second loop: run remaining jobA and two jobC
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         manager.schedule_script.reset_mock()
         result = next(manager._run_loops())
         self.assertTrue(result)
@@ -1937,7 +1938,7 @@ class ManagerTest(BaseTestCase):
             result = next(manager._run_loops())
             self.assertTrue(result)
             self.assertEqual(manager.schedule_script.call_count, len(side_effect))
-            manager.is_finished = lambda x: "finished"
+            manager.is_finished = lambda x: Outcome("finished")
             manager.schedule_script.reset_mock()
         result = next(manager._run_loops())
         self.assertFalse(result)
@@ -1957,7 +1958,7 @@ class ManagerTest(BaseTestCase):
         result = next(manager._run_loops())
         self.assertTrue(result)
         self.assertEqual(manager.schedule_script.call_count, 2)
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
 
         result = next(manager._run_loops())
         self.assertFalse(result)
@@ -2084,7 +2085,7 @@ class ManagerTest(BaseTestCase):
 
         # third loop: jobA finishes, can run jobB
         manager.schedule_script.side_effect = ["999/2/1"]
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         self.assertTrue(next(manager._run_loops()))
         self.assertEqual(manager.schedule_script.call_count, 5)
         manager.schedule_script.assert_called_with(["commandB"], tags=["TASK_ID=jobB"], units=None, project_id=None)
@@ -2150,7 +2151,7 @@ class ManagerTest(BaseTestCase):
 
             def on_start(self):
                 jobB = Task(task_id="jobB", command="commandB --optionB=B")
-                self.get_task("jobA").add_next_task(jobB)
+                self.get_task(TaskId("jobA")).add_next_task(jobB)
                 super().on_start()
 
         mocked_get_jobs.side_effect = [[]]
@@ -2160,7 +2161,7 @@ class ManagerTest(BaseTestCase):
         manager.schedule_script = Mock()
         manager.schedule_script.side_effect = ["999/1/1"]
         manager._on_start()
-        self.assertEqual(manager.jobs_graph["jobA"]["on_finish"]["default"], ["jobB"])
+        self.assertEqual(manager.jobs_graph[TaskId("jobA")]["on_finish"]["default"], ["jobB"])
 
         # first loop
         self.assertTrue(next(manager._run_loops()))
@@ -2174,7 +2175,7 @@ class ManagerTest(BaseTestCase):
 
         # second loop
         manager.schedule_script.side_effect = ["999/2/1"]
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         self.assertTrue(next(manager._run_loops()))
         self.assertEqual(manager.schedule_script.call_count, 2)
 
@@ -2209,7 +2210,7 @@ class ManagerTest(BaseTestCase):
         manager.schedule_script = Mock()
         manager.schedule_script.side_effect = ["999/1/1"]
         manager._on_start()
-        self.assertEqual(manager.jobs_graph["jobB"]["on_finish"]["default"], [])
+        self.assertEqual(manager.jobs_graph[TaskId("jobB")]["on_finish"]["default"], [])
         self.assertEqual(manager.start_callback_count, 0)
 
         # first loop
@@ -2221,23 +2222,23 @@ class ManagerTest(BaseTestCase):
             units=None,
             project_id=None,
         )
-        self.assertEqual(manager.jobs_graph["jobB"]["on_finish"]["default"], [])
+        self.assertEqual(manager.jobs_graph[TaskId("jobB")]["on_finish"]["default"], [])
         self.assertEqual(manager.start_callback_count, 0)
 
         # second loop, jobB is started, so start callback is called
         manager.schedule_script.side_effect = ["999/2/1"]
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         self.assertTrue(next(manager._run_loops()))
         self.assertEqual(manager.schedule_script.call_count, 2)
         manager.schedule_script.assert_any_call(
             ["commandB", "--optionB=B"], tags=["TASK_ID=jobB"], units=None, project_id=None
         )
-        self.assertEqual(manager.jobs_graph["jobB"]["on_finish"]["default"], ["jobC"])
+        self.assertEqual(manager.jobs_graph[TaskId("jobB")]["on_finish"]["default"], ["jobC"])
         self.assertEqual(manager.start_callback_count, 1)
 
         # second loop, jobB is started, so start callback is called
         manager.schedule_script.side_effect = ["999/3/1"]
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         self.assertTrue(next(manager._run_loops()))
         self.assertEqual(manager.schedule_script.call_count, 3)
         manager.schedule_script.assert_any_call(
@@ -2268,7 +2269,7 @@ class ManagerTest(BaseTestCase):
         manager.schedule_script = Mock()
         manager.schedule_script.side_effect = ["999/1/1"]
         manager._on_start()
-        self.assertEqual(manager.jobs_graph["jobB"]["on_finish"]["default"], [])
+        self.assertEqual(manager.jobs_graph[TaskId("jobB")]["on_finish"]["default"], [])
 
         # first loop
         self.assertTrue(next(manager._run_loops()))
@@ -2279,11 +2280,11 @@ class ManagerTest(BaseTestCase):
             units=None,
             project_id=None,
         )
-        self.assertEqual(manager.jobs_graph["jobB"]["on_finish"]["default"], [])
+        self.assertEqual(manager.jobs_graph[TaskId("jobB")]["on_finish"]["default"], [])
 
         # second loop, jobB is started, so start callback is called
         manager.schedule_script.side_effect = ["999/2/1"]
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         self.assertTrue(next(manager._run_loops()))
         self.assertEqual(manager.schedule_script.call_count, 2)
         manager.schedule_script.assert_any_call(
@@ -2364,7 +2365,7 @@ class ManagerTest(BaseTestCase):
         self.assertEqual(manager.schedule_script.call_count, 0)
 
         # second loop. jobA remaining jobs completed. Continue with jobB and jobC
-        manager.is_finished = lambda x: "finished"
+        manager.is_finished = lambda x: Outcome("finished")
         manager.schedule_script.side_effect = [
             "999/2/1",
             "999/2/2",
